import pandas as pd
from ..utils import *
from ..path import *


def calculate_prf(df, ground_truth_colunm, predict_column, breakdown_column=None, is_drop_na=True):
	"""
	Calculate the precision, recall, F1 of the prediction
	:param df: input dataframe
	:param ground_truth_colunm: ground truth column
	:param predict_column: predict column
	:param breakdown_column: breakdown column
	:param is_drop_na: drop the rows with missing values in the ground_truth_colunm and predict_column
	"""
	# # check df is a pandas dataframe, ground_truth_colunm and predict_column, breakdown_column are in the df
	# assert isinstance(df, pd.DataFrame), "df should be a pandas dataframe"
	# assert ground_truth_colunm in df.columns, f"{ground_truth_colunm} should be in the dataframe"
	# assert predict_column in df.columns, f"{predict_column} should be in the dataframe"
	# if breakdown_column is not None:
	# 	assert breakdown_column in df.columns, f"{breakdown_column} should be in the dataframe"
	
	# remove rows with missing values in the two columns
	if is_drop_na:
		df = drop_na(df, [ground_truth_colunm, predict_column])

	# check the value of ground_truth_colunm and predict_column are in [0, 1]
	assert df[ground_truth_colunm].isin([0, 1]).all(), f"{ground_truth_colunm} should only contain 0 or 1"
	assert df[predict_column].isin([0, 1]).all(), f"{predict_column} should only contain 0 or 1"
	
	path = get_output_dir()
	if breakdown_column is None:
		prf_df = calculate_prf_single(df, ground_truth_colunm, predict_column)
		prf_df = prf_df.to_frame().T
		save_df_to_csv(prf_df, path + f'/{ground_truth_colunm}_{predict_column}_prf.csv')
	else:
		prf_df = df.groupby(breakdown_column).apply(lambda x: calculate_prf_single(x, ground_truth_colunm, predict_column)).reset_index()
		save_df_to_csv(prf_df, path + f'/{ground_truth_colunm}_{predict_column}_breakdown_{breakdown_column}_prf.csv')
	return prf_df


def calculate_prf_single(df, ground_truth_colunm, predict_column):
	"""
	Calculate the precision, recall, F1 of the prediction for a single group
	:param df: input dataframe
	:param ground_truth_colunm: ground truth column
	:param predict_column: predict column
	"""
	ground_truth = df[ground_truth_colunm].values.tolist()
	predict = df[predict_column].values.tolist()
	TP = 0
	FP = 0
	FN = 0
	for i in range(len(ground_truth)):
		if ground_truth[i] == 1 and predict[i] == 1:
			TP += 1
		elif ground_truth[i] == 0 and predict[i] == 1:
			FP += 1
		elif ground_truth[i] == 1 and predict[i] == 0:
			FN += 1
	TN = len(ground_truth) - TP - FP - FN
	if TP + FP == 0:
		precision = 0
	else:
		precision = TP / (TP + FP)
	if TP + FN == 0:
		recall = 0
	else:
		recall = TP / (TP + FN)
	if precision + recall == 0:
		f1 = 0
	else:
		f1 = 2 * precision * recall / (precision + recall)
	ground_truth_positive = TP + FN
	ground_truth_positive_ratio = ground_truth_positive / len(ground_truth)
	predict_positive = TP + FP
	predict_positive_ratio = predict_positive / len(predict)
	FPR = FP / (FP + TN)
	return pd.Series({'TP': TP, 'FP': FP, 'TN': TN, 'FN': FN, 'precision': precision, 'recall': recall, 'f1': f1, 'FPR': FPR,
				   'ground_truth_positive': ground_truth_positive, 'ground_truth_positive_ratio': ground_truth_positive_ratio, 
				   'predict_positive': predict_positive, 'predict_positive_ratio': predict_positive_ratio
				   })
	


import pandas as pd

def save_df_to_csv(df, path):
	df.to_csv(path, index=False)
	print(f">>>> save dataframe result to {path}")


def drop_na(df, columns):
	"""
	Drop the rows with na value in the columns
	:param df: DataFrame
	:param columns: list of column names
	:return: DataFrame
	"""
	all_rows = df.shape[0]
	df = df.dropna(subset=columns)
	drop_rows = all_rows - df.shape[0]
	print(f'>>>> all rows = {all_rows}, drop rows = {drop_rows} in {columns}')
	return df


import os
from os import environ, listdir
from os.path import join

import pandas as pd


def load_file(file_path, sep=None):
	"""
	Load the file
	:param file_path: the file path
	:param sep: the separator of the file
	:return: the dataframe
	"""
	if sep is None:
		with open(file_path, 'r', encoding="utf8") as f:
			first_line = f.readline()
			if '\t' in first_line:
				sep = '\t'
			elif ',' in first_line:
				sep = ','
			else:
				raise ValueError('The file does not contain the valid separator')
	df = pd.read_csv(file_path, sep=sep)
	return df


def load_dir(dir_path, target_str=None, sep=None):
	"""
	Load all files in the directory with the target string in the file name
	:param dir_path: the directory path
	:param target_str: the target string in the file name
	:param sep: the separator of the file
	:return: the concatenated dataframe
	"""
	if target_str is not None:
		file_paths = [join(dir_path, f) for f in listdir(dir_path) if target_str in f]
	else:
		file_paths = [join(dir_path, f) for f in listdir(dir_path)]
	dfs = []
	for file_path in file_paths:
		dfs.append(load_file(file_path, sep))
	df = pd.concat(dfs)
	df = df.reset_index(drop=True)
	return df




import os
import pandas as pd
import numpy as np
from ..visualization import *
from ..path import *
from ..utils import *

COUNT = 'Count'
RATIO = 'Ratio'
SHIFT = 'Shift'
GROUP_TOTAL = 'group_total'
GROUP_RATIO = 'group_ratio'
TOTAL_RATIO = 'total_ratio'
COLUMN1_COLUMN2 = 'column1_column2'

def single_feature_analysis(df, column, is_drop_na=True):
	"""
	Analyze the single feature
	:param df: DataFrame
	:param column: column name
	:param is_drop_na: drop the rows with no value
	:return: analysis
	"""
	path = get_output_dir()
	bar_graph_path = path + f'/{column}_bar_plot.png'
	pie_graph_path = path + f'/{column}_pie_plot.png'
	distribution_result_path = path + f'/{column}_distribution.csv'
	
	if is_drop_na:
		df = drop_na(df, [column])
	feature_analysis = df.groupby(column)[column].value_counts().reset_index(name=COUNT)
	total_count = feature_analysis[COUNT].sum()
	feature_analysis[RATIO] = feature_analysis[COUNT] / total_count

	bar_plot(data=feature_analysis, xlabel=column, ylabel=COUNT, title=f'Distribution of {column}', path=bar_graph_path)
	pie_plot(data=feature_analysis, labels=feature_analysis[column], sizes=feature_analysis[COUNT], title=f'Distribution of {column}', path=pie_graph_path)

	save_df_to_csv(feature_analysis, distribution_result_path)
	return feature_analysis


def pivot_table(df, column1, column2, is_drop_na=True):
	"""
	Create a pivot table for two columns
	:param df: DataFrame
	:param column1: column name
	:param column2: column name
	:param is_drop_na: drop the rows with no value
	:param path: path to save the result
	:return: analysis
	"""

	
	path = get_output_dir()
	pivot_count_path = path + f'/{column1}_{column2}_pivot_count.png'
	pivot_group_ratio_path = path + f'/{column1}_{column2}_pivot_group_ratio.png'
	pivot_total_ratio_path = path + f'/{column1}_{column2}_pivot_total_ratio.png'
	pivot_result_path = path + f'/{column1}_{column2}_pivot_table.csv'

	if is_drop_na:
		df = drop_na(df, [column1, column2])

	pivot_analysis = df.groupby([column1, column2]).size().reset_index(name=COUNT)
	pivot_analysis = pivot_analysis.sort_values(by=COUNT, ascending=False)
	total_count = pivot_analysis[COUNT].sum()
	pivot_analysis[TOTAL_RATIO] = pivot_analysis[COUNT] / total_count

	column1_df = pivot_analysis.groupby([column1])[COUNT].sum().reset_index()
	column1_df = column1_df.rename(columns={COUNT: GROUP_TOTAL})
	pivot_analysis = pd.merge(pivot_analysis, column1_df, on=column1, how='left')
	pivot_analysis[GROUP_RATIO] = pivot_analysis[COUNT] / pivot_analysis[GROUP_TOTAL]

	group_pivot_analysis = pivot_analysis.copy()
	group_pivot_analysis[COLUMN1_COLUMN2] = group_pivot_analysis[column1].astype(str) + "&" + group_pivot_analysis[column2].astype(str)
	pivot_bar_plot(data=pivot_analysis, index=column1, columns=column2, values=COUNT, title=f'Count of {column1} and {column2}', path=pivot_count_path)
	pivot_bar_plot(data=pivot_analysis, index=column1, columns=column2, values=GROUP_RATIO, title=f'Group Ratio  {column2}', path=pivot_group_ratio_path)
	pie_plot(data=group_pivot_analysis, labels=group_pivot_analysis[COLUMN1_COLUMN2], sizes=group_pivot_analysis[COUNT], title=f'Total Ratio of {column1} + {column2}', path=pivot_total_ratio_path)

	save_df_to_csv(pivot_analysis, pivot_result_path)
	return pivot_analysis


def feature_shift_analysis(df, column1, column2, gen_pivot_table=True, is_drop_na=True):
	"""
	Analyze the shift of two columns
	:param df: DataFrame
	:param column1: column name
	:param column2: column name
	:param gen_pivot_table: generate pivot table
	:param is_drop_na: drop the rows with no value
	:param path: path to save the result
	:return: analysis
	"""
	if gen_pivot_table:
		pivot_table(df, column1, column2, is_drop_na)

	path = get_output_dir()
	overall_shift_graph_path = path + f'/{column1}_{column2}_shift_ratio_plot.png'
	overall_shift_path = path + f'/{column1}_{column2}_shift_table.csv'

	if is_drop_na:
		df = drop_na(df, [column1, column2])

	df[SHIFT] = df[column1] != df[column2]
	overall_shift = df[SHIFT].value_counts().reset_index(name=COUNT)
	overall_shift[RATIO] = overall_shift[COUNT] / overall_shift[COUNT].sum()
	pie_plot(data=overall_shift, labels=overall_shift[SHIFT], sizes=overall_shift[COUNT], title=f'Shift of {column1} and {column2}', path=overall_shift_graph_path)
	save_df_to_csv(overall_shift, overall_shift_path)

	return overall_shift


def feature_correlation_coefficient(df, column, is_drop_na=True):
	"""
	Calculate the correlation coefficient of the column with other columns
	:param df: DataFrame
	:param column: column name
	:param is_drop_na: drop the rows with no value
	:return: correlation coefficient
	"""
	if is_drop_na:
		df = drop_na(df, [column])
	
	# filter the columns with numeric values
	df = df.select_dtypes(include=[np.number])
	
	path = get_output_dir()
	correlation_coefficient_path = path + f'/{column}_correlation_coefficient.csv'

	correlation_coefficient = df.corr()[column].reset_index()
	correlation_coefficient = correlation_coefficient.rename(columns={column: 'correlation_coefficient'})
	correlation_coefficient = correlation_coefficient.sort_values(by='correlation_coefficient', ascending=False)
	save_df_to_csv(correlation_coefficient, correlation_coefficient_path)
	return correlation_coefficient


import numpy as np
import seaborn as sns
import matplotlib.pyplot as plt

def bar_plot(data, xlabel=None, ylabel=None, title=None, path=None):
	"""Plot a bar plot."""
	# show number on the bar
	ax = sns.barplot(x=xlabel, y=ylabel, data=data)
	for p in ax.patches:
		p.set_height(p.get_height())
		ax.text(p.get_x() + p.get_width() / 2., p.get_height(), '%d' % int(p.get_height()), ha='center', va='bottom')
	plt.title(title)
	if path:
		plt.savefig(path)
		print(f">>>> Plot saved at {path}")
	else:
		print(">>>> No path provided, plot will not be saved.")


def pie_plot(data, labels=None, sizes=None, title=None, path=None):
	"""Plot a pie plot."""
	fig1, ax1 = plt.subplots()
	ax1.pie(sizes, labels=labels, autopct='%1.1f%%', shadow=True, startangle=90)
	ax1.axis('equal')
	plt.title(title)
	if path:
		plt.savefig(path)
		print(f">>>> Plot saved at {path}")
	else:
		print(">>>> No path provided, plot will not be saved.")


def pivot_bar_plot(data, index=None, columns=None, values=None, title=None, path=None):
	"""Plot a bar plot."""
	# show number on the bar
	data = data.pivot(index=index, columns=columns, values=values)
	ax = data.plot(kind='bar', stacked=False)
	for p in ax.patches:
		p.set_height(p.get_height())
		if 'ratio' in values.lower():
			ax.text(p.get_x() + p.get_width() / 2., p.get_height(), '%d%%' % int(p.get_height() * 100), ha='center', va='bottom')
		else:
			ax.text(p.get_x() + p.get_width() / 2., p.get_height(), '%d' % int(p.get_height()), ha='center', va='bottom')
	plt.title(title)
	if path:
		plt.savefig(path)
		print(f">>>> Plot saved at {path}")
	else:
		print(">>>> No path provided, plot will not be saved.")



if __name__ == '__main__':
	import pandas as pd
	data = {'Market': ['en_us', 'en_us', 'zh_cn'], 'Prediction': [0, 1, 0], 'Count': [1, 1, 2]}
	df = pd.DataFrame(data)
	# single_bar_plot(df, 'Prediction', 'Count', 'Distribution of Prediction', 'Prediction_distribution.png')
	pivot_bar_plot(df, 'Market', 'Prediction', 'Distribution of Prediction', 'Prediction_distribution.png')


import os
import time

from .datasets import (
	load_dir,
	load_file
)

from .metrics import (
	calculate_prf,
)

from .statistics import (
	single_feature_analysis,
	pivot_table,
	feature_shift_analysis,
	feature_correlation_coefficient,
)

from .path import (
	set_output_dir,
	get_output_dir,
)


from .sample import (
	custom_sample,
)





from setuptools import setup, find_packages

VERSION = '1.0.6' 
DESCRIPTION = 'Reco evaluation tool'
LONG_DESCRIPTION = 'This tool aims to provide a set of evaluation metrics for recommendation'

setup(
        name="reco_eval_tool", 
        version=VERSION,
        author="Minglei Guo",
        author_email="<mingleiguo@microsoft.com>",
        description=DESCRIPTION,
        long_description=LONG_DESCRIPTION,
        packages=find_packages(),
        install_requires=[
			"pandas",
			"numpy",
			"seaborn",
			"matplotlib",				  
        ],         
        keywords=['evaluation', 'precision', 'recall', 'F1'],
        classifiers= [
            "Development Status :: 3 - Alpha",
            "Intended Audience :: Education",
            "Programming Language :: Python :: 3",
            "Operating System :: MacOS :: MacOS X",
            "Operating System :: Microsoft :: Windows",
        ]
)


import sys
from pathlib import Path
sys.path.append(str(Path(__file__).resolve().parents[2])) 

import reco_eval_tool


demo_path = Path.joinpath(Path(__file__).resolve().parents[0], 'demo')
file1_path = Path.joinpath(demo_path, 'file1.tsv')

df = reco_eval_tool.datasets.load_file(file1_path)
print(df)
assert df.shape == (2, 2)

df = reco_eval_tool.datasets.load_dir(demo_path)
print(df)
assert df.shape == (4, 2)
